/***********************************************************
 * find_edges.c
 * written by Steve Sprang
 * Copyright  1997
 *
 * Aurora v0.2A
 *
 * silly find edges
 * Uses slow blurring...
 * Just wanted to get some filters up fast...
 **********************************************************/

#include "filter.h"

char name[] = "Find Edges";

void aurora_name(char * &string)
{
	string = name;
}

bool aurora_filter(BBitmap *input, BView *painter)
{
	if(input->ColorSpace() != B_RGB_32_BIT) {
		return false;
	}
	else {
		BRect		bounds = input->Bounds();
		BBitmap		*diff = new BBitmap(bounds, B_RGB_32_BIT, true);
		BView		*vw = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
		
		diff->AddChild(vw);
		diff->Lock();
		vw->DrawBitmapAsync(input);
		vw->Sync();
		diff->Unlock();
		
		uchar 		*srcbits = (uchar *) input->Bits();
		uchar 		*destbits = (uchar *) diff->Bits();
		long		xmod = input->BytesPerRow();
		int			x,y, width, height, sum, x_prev, x_curr, x_next;
		
		width = bounds.Width();
		height = bounds.Height();
		
		srcbits += xmod;
		destbits += xmod;
		
		// slow blur (redundant additions)...
		// also ignores 1 pixel border around edge...
		// Alpha Paint App => Alpha Filters ;)
		for(y = 1; y < height; y++) {
			for(x = 1; x < width; x++) {
				for(int channel = 0; channel < 3; channel++) {
					x_curr = x * 4 + channel;
					x_prev = x_curr - 4;
					x_next = x_curr + 4;
					
					sum = (	(int) srcbits[x_next] +
							(int) srcbits[x_prev] +
								 		
							(int) srcbits[x_prev - xmod] +
							(int) srcbits[x_curr - xmod] +
							(int) srcbits[x_next - xmod] +
								 		
							(int) srcbits[x_prev + xmod] +
							(int) srcbits[x_curr + xmod] +
							(int) srcbits[x_next + xmod]) >> 3;
					
					// screwy average... but fast dividin'!	
					destbits[x*4 + channel] = (srcbits[x*4 + channel] + sum) >> 1;
				}
			}
			destbits += xmod;
			srcbits += xmod;
		}
		
		diff->Lock();
		vw->SetDrawingMode(B_OP_SUBTRACT);
		vw->DrawBitmap(input); // subtract original image from blur
		
		// quaduple the diff image for emphasis
		vw->SetDrawingMode(B_OP_ADD);
		vw->DrawBitmap(diff);
		vw->DrawBitmap(diff);
		vw->DrawBitmap(diff);
		vw->Sync();
		diff->Unlock();
		
		input->Lock();
		painter->DrawBitmap(diff);
		painter->InvertRect(painter->Bounds());
		painter->Sync();
		input->Unlock();
		
		delete diff;
	}
	return true;
}